@@ -1,9 +1,8 @@
-import abc
 import time
 import warnings
 from copy import deepcopy
 from functools import partial, wraps
-from typing import Any, Callable, Dict, List, Optional
+from typing import Any, Callable, Dict, List, Optional, Union
 
 from deepdiff import DeepHash
 
@@ -11,7 +10,7 @@ from lightning_app.core.queues import BaseQueue
 from lightning_app.storage import Path
 from lightning_app.storage.drive import _maybe_create_drive, Drive
 from lightning_app.storage.payload import Payload
-from lightning_app.utilities.app_helpers import _is_json_serializable, _LightningAppRef
+from lightning_app.utilities.app_helpers import _is_json_serializable, _LightningAppRef, is_overridden
 from lightning_app.utilities.component import _is_flow_context, _sanitize_state
 from lightning_app.utilities.enum import (
     CacheCallsKeys,
@@ -25,11 +24,16 @@ from lightning_app.utilities.exceptions import LightningWorkException
 from lightning_app.utilities.introspection import _is_init_context
 from lightning_app.utilities.network import find_free_network_port
 from lightning_app.utilities.packaging.build_config import BuildConfig
-from lightning_app.utilities.packaging.cloud_compute import CloudCompute
+from lightning_app.utilities.packaging.cloud_compute import (
+    _CLOUD_COMPUTE_STORE,
+    _CloudComputeStore,
+    _maybe_create_cloud_compute,
+    CloudCompute,
+)
 from lightning_app.utilities.proxies import LightningWorkSetAttrProxy, ProxyWorkRun, unwrap
 
 
-class LightningWork(abc.ABC):
+class LightningWork:
 
     _INTERNAL_STATE_VARS = (
         # Internal protected variables that are still part of the state (even though they are prefixed with "_")
@@ -86,7 +90,7 @@ class LightningWork(abc.ABC):
             :header: The Lightning Work inner workings.
             :description: Learn more Lightning Work.
             :col_css: col-md-4
-            :button_link: ../../../core_api/lightning_work/index.html
+            :button_link: ../../core_api/lightning_work/index.html
             :height: 180
             :tag: Basic
 
@@ -104,7 +108,7 @@ class LightningWork(abc.ABC):
                 " in the next version. Use `cache_calls` instead."
             )
         self._cache_calls = run_once if run_once is not None else cache_calls
-        self._state = {"_host", "_port", "_url", "_future_url", "_internal_ip", "_restarting"}
+        self._state = {"_host", "_port", "_url", "_future_url", "_internal_ip", "_restarting", "_cloud_compute"}
         self._parallel = parallel
         self._host: str = host
         self._port: Optional[int] = port
@@ -139,6 +143,7 @@ class LightningWork(abc.ABC):
         self._cloud_build_config = cloud_build_config or BuildConfig()
         self._cloud_compute = cloud_compute or CloudCompute()
         self._backend: Optional[Backend] = None
+        self._check_run_is_implemented()
         self._on_init_end()
 
     @property
@@ -226,8 +231,14 @@ class LightningWork(abc.ABC):
         return self._cloud_compute
 
     @cloud_compute.setter
-    def cloud_compute(self, cloud_compute) -> None:
+    def cloud_compute(self, cloud_compute: CloudCompute) -> None:
         """Returns the cloud compute used to select the cloud hardware."""
+        # A new ID
+        current_id = self._cloud_compute.id
+        new_id = cloud_compute.id
+        if current_id != new_id:
+            compute_store: _CloudComputeStore = _CLOUD_COMPUTE_STORE[current_id]
+            compute_store.remove(self.name)
         self._cloud_compute = cloud_compute
 
     @property
@@ -316,9 +327,17 @@ class LightningWork(abc.ABC):
 
         return has_succeeded_counter
 
+    def _get_property_if_exists(self, name: str) -> Union[property, None]:
+        attr = getattr(self.__class__, name, None)
+        return attr if isinstance(attr, property) else None
+
     def __setattr__(self, name: str, value: Any) -> None:
-        setattr_fn = getattr(self, "_setattr_replacement", None) or self._default_setattr
-        setattr_fn(name, value)
+        property_object = self._get_property_if_exists(name)
+        if property_object is not None and property_object.fset is not None:
+            property_object.fset(self, value)
+        else:
+            setattr_fn = getattr(self, "_setattr_replacement", None) or self._default_setattr
+            setattr_fn(name, value)
 
     def _default_setattr(self, name: str, value: Any) -> None:
         from lightning_app.core.flow import LightningFlow
@@ -480,6 +499,8 @@ class LightningWork(abc.ABC):
         for k, v in provided_state["vars"].items():
             if isinstance(v, Dict):
                 v = _maybe_create_drive(self.name, v)
+            if isinstance(v, Dict):
+                v = _maybe_create_cloud_compute(v)
             setattr(self, k, v)
 
         self._changes = provided_state["changes"]
@@ -516,14 +537,12 @@ class LightningWork(abc.ABC):
                         final_statuses.append(status)
                 calls[call_hash]["statuses"] = final_statuses
 
-    @abc.abstractmethod
     def run(self, *args, **kwargs):
         """Override to add your own logic.
 
         Raises:
             LightningPlatformException: If resource exceeds platform quotas or other constraints.
         """
-        pass
 
     def on_exception(self, exception: BaseException):
         """Override to customize how to handle exception in the run method."""
@@ -544,10 +563,6 @@ class LightningWork(abc.ABC):
         status = {**timeout_statuses[-1], "timestamp": statuses[0]["timestamp"]}
         return WorkStatus(**status, count=len(timeout_statuses))
 
-    def load_state_dict(self, state):
-        # TODO (tchaton) Implement logic for state reloading.
-        pass
-
     def on_exit(self):
         """Override this hook to add your logic when the work is exiting."""
         pass
@@ -566,3 +581,17 @@ class LightningWork(abc.ABC):
         self._calls[latest_hash]["statuses"].append(stop_status)
         app = _LightningAppRef().get_current()
         self._backend.stop_work(app, self)
+
+    def _check_run_is_implemented(self) -> None:
+        if not is_overridden("run", instance=self, parent=LightningWork):
+            raise TypeError(
+                f"The work `{self.__class__.__name__}` is missing the `run()` method. This is required. Implement it"
+                " first and then call it in your Flow."
+            )
+
+    def _register_cloud_compute(self):
+        internal_id = self.cloud_compute.id
+        assert internal_id
+        if internal_id not in _CLOUD_COMPUTE_STORE:
+            _CLOUD_COMPUTE_STORE[internal_id] = _CloudComputeStore(id=internal_id, component_names=[])
+        _CLOUD_COMPUTE_STORE[internal_id].add_component_name(self.name)
