@@ -1,11 +1,13 @@
 from abc import ABC, abstractmethod
 from functools import partial
-from typing import Any, Callable, List, Optional
+from typing import Any, Callable, List, Optional, TYPE_CHECKING
 
-import lightning_app
 from lightning_app.core.queues import QueuingSystem
 from lightning_app.utilities.proxies import ProxyWorkRun, unwrap
 
+if TYPE_CHECKING:
+    import lightning_app
+
 
 class Backend(ABC):
     """The Backend provides and interface for the framework to communicate with resources in the cloud."""
@@ -78,14 +80,14 @@ class Backend(ABC):
 
         work.run = partial(self._dynamic_run_wrapper, app=app, work=work, work_run=unwrap(work.run))
 
-    def _prepare_queues(self, app):
+    def _prepare_queues(self, app: "lightning_app.LightningApp"):
         kw = dict(queue_id=self.queue_id)
         app.delta_queue = self.queues.get_delta_queue(**kw)
         app.readiness_queue = self.queues.get_readiness_queue(**kw)
         app.api_response_queue = self.queues.get_api_response_queue(**kw)
         app.error_queue = self.queues.get_error_queue(**kw)
         app.api_publish_state_queue = self.queues.get_api_state_publish_queue(**kw)
-        app.api_delta_queue = self.queues.get_api_delta_queue(**kw)
+        app.api_delta_queue = app.delta_queue
         app.request_queues = {}
         app.response_queues = {}
         app.copy_request_queues = {}
